@@ -29,7 +29,7 @@
  *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  Last-modified: Wed 16 Jul 2008 18:50:03 JST
+ *  Last-modified: Wed 22 Oct 2008 14:50:00 JST
  */
 /* ------------------------------------------------------------------------- */
 #ifndef CLX_TCP_H
@@ -37,6 +37,7 @@
 
 #include <string>
 #include <stdexcept>
+#include "resolver.h"
 #include "socket.h"
 #include "sockaddress.h"
 #include "sockstream.h"
@@ -57,75 +58,70 @@ namespace clx {
 			typedef typename std::basic_string<char> string_type;
 			
 			// constructor and destructor
-			basic_socket() : rawsocket(), addr_() {}
+			basic_socket() : rawsocket(), addr_(), connect_(false) {}
 			
-			basic_socket(const basic_socket& cp) : addr_(cp.addr_) {
+			basic_socket(const basic_socket& cp) : addr_(cp.addr_), connect_(cp.connect_) {
 				this->assign(cp);
 			}
 			
 			basic_socket& operator=(const basic_socket& cp) {
 				addr_ = cp.addr_;
+				connect_ = cp.connect_;
 				this->assign(cp);
 			}
 			
 			explicit basic_socket(socket_int s, const sockaddress& addr) :
-				rawsocket(s), addr_(addr) {}
+				rawsocket(s), addr_(addr), connect_(true) {}
 			
 			explicit basic_socket(const char_type* host, int port) :
-				rawsocket(), addr_(host, port) { this->connect(); }
+				rawsocket(), addr_(host, port), connect_(false) { this->xconnect(); }
 			
 			explicit basic_socket(const string_type& host, int port) :
-				rawsocket(), addr_(host, port) { this->connect(); }
-			
-			explicit basic_socket(const char_type* host, const char_type* service) :
-				rawsocket(), addr_(host, service) { this->connect(); }
-			
-			explicit basic_socket(const string_type& host, const string_type& service) :
-				rawsocket(), addr_(host, service) { this->connect(); }
+				rawsocket(), addr_(host, port), connect_(false) { this->xconnect(); }
 			
 			virtual ~basic_socket() {}
 			
 			// operator
 			basic_socket& connect(const char_type* host, int port) {
 				addr_.assign(host, port);
-				return this->connect();
+				return this->xconnect();
 			}
 			
 			basic_socket& connect(const string_type& host, int port) {
 				return this->connect(host.c_str(), port);
 			}
 			
-			basic_socket& connect(const char_type* host, const char_type* service) {
-				addr_.assign(host, service);
-				return this->connect();
+			virtual void close() {
+				rawsocket::close();
+				connect_ = false;
 			}
 			
-			basic_socket& connect(const string_type& host, const string_type& service) {
-				return this->connect(host.c_str(), service.c_str());
+			virtual int send(const char_type* src, int n) {
+				return ::send(this->socket(), src, n, 0);
 			}
 			
-			virtual int send(const char_type* dat, int n) {
-				return ::send(this->socket(), dat, n, 0);
+			virtual int send(const string_type& src) {
+				return this->send(src.c_str(), static_cast<int>(src.size()));
 			}
 			
-			virtual int send(const string_type& dat) {
-				return this->send(dat.c_str(), static_cast<int>(dat.size()));
-			}
-			
-			virtual int recv(char_type* buf, int n) {
-				return ::recv(this->socket(), buf, n, 0);
+			virtual int recv(char_type* dest, int n) {
+				return ::recv(this->socket(), dest, n, 0);
 			}
 			
 			const sockaddress& address() const { return addr_; }
-			
+			bool is_connect() const { return connect_; }
 		private:
 			sockaddress addr_;
+			bool connect_;
 			
-			basic_socket& connect() {
+			basic_socket& xconnect() {
+				if (connect_) return *this;
 				if (this->open() < 0) throw clx::socket_error("socket");
 				if (::connect(this->socket(), (struct sockaddr*)addr_.data(), addr_.size()) == -1) {
 					throw clx::socket_error("connect");
 				}
+				connect_ = true;
+				
 				return *this;
 			}
 		};
@@ -141,46 +137,30 @@ namespace clx {
 			typedef char char_type;
 			typedef typename std::basic_string<char> string_type;
 			
-			static const int default_nlisten = 100;
+			enum { nmax = 100 };
 			
 			// constructor and destructor
-			basic_acceptor() : rawsocket(), nlisten_(default_nlisten) {}
+			basic_acceptor() : rawsocket(), nlisten_(nmax) {}
 			
-			explicit basic_acceptor(int port, int n = default_nlisten) :
-				rawsocket(), addr_(port), nlisten_(default_nlisten) { this->open(); }
-			
-			explicit basic_acceptor(const char_type* service, int n = default_nlisten) :
-				rawsocket(), addr_(service), nlisten_(default_nlisten) { this->open(); }
-			
-			explicit basic_acceptor(const string_type& service, int n = default_nlisten) :
-				rawsocket(), addr_(service), nlisten_(default_nlisten) { this->open(); }
+			explicit basic_acceptor(int port, int n = nmax) :
+				rawsocket(), addr_(port), nlisten_(nmax) { this->xopen(); }
 			
 			virtual ~basic_acceptor() {}
 			
 			// operator
-			basic_acceptor& open(int port, int n = default_nlisten) {
+			basic_acceptor& open(int port, int n = nmax) {
 				addr_.assign(port);
 				nlisten_ = n;
-				return this->open();
+				return this->xopen();
 			}
 			
-			basic_acceptor& open(const char_type* service, int n = default_nlisten) {
-				addr_.assign(service);
-				nlisten_ = n;
-				return this->open();
-			}
-			
-			basic_acceptor& open(const string_type& service, int n = default_nlisten) {
-				return this->open(service.c_str(), n);
-			}
-			
-			basic_socket<Family> accept(int retry = 10) {
+			basic_socket<Family> accept() {
 				socket_int s = -1;
 				sockaddress addr;
 				typename sockaddress::inet_type tmp;
 				socklen_type len = sizeof(tmp);
 				
-				for (int i = 0; i < retry; i++) {
+				for (int foo = 0; foo < 10; ++foo) {
 					s = ::accept(this->socket(), (struct sockaddr*)&tmp, &len);
 					if (s == -1) clx::sleep(1.0);
 					else break;
@@ -193,7 +173,7 @@ namespace clx {
 			sockaddress addr_;
 			int nlisten_;
 			
-			basic_acceptor& open() {
+			basic_acceptor& xopen() {
 				if (rawsocket::open() < 0) throw socket_error("socket");
 				if (::bind(this->socket(), (struct sockaddr*)addr_.data(), addr_.size()) == -1) {
 					throw socket_error("bind");
@@ -203,10 +183,22 @@ namespace clx {
 			}
 		};
 		
+		/* ----------------------------------------------------------------- */
+		//  port
+		/* ----------------------------------------------------------------- */
+		int port(const char* service) {
+			return clx::port(service, "tcp");
+		}
+		
+		int port(const std::basic_string<char>& service) {
+			return clx::tcp::port(service.c_str());
+		}
+		
 		typedef basic_rawsocket<SOCK_STREAM, AF_INET> rawsocket;
 		typedef basic_socket<AF_INET> socket;
 		typedef basic_acceptor<AF_INET> acceptor;
 		typedef basic_sockaddress<AF_INET, IPPROTO_TCP> sockaddress;
+		typedef basic_sockbuf<socket> sockbuf;
 		typedef basic_sockstream<socket> sockstream;
 		typedef basic_sockmanager<SOCK_STREAM, AF_INET> sockmanager;
 	};
